declare enum Direction {
    FORWARD = 0,
    BACKWARD = 1
}
export declare enum ValueType {
    REAL = 0,
    COMPLEX = 1
}
type FloatArray = Float32Array | Float64Array;
type FloatArrayType<A extends FloatArray> = A extends Float32Array ? Float32ArrayConstructor : Float64ArrayConstructor;
type PFFFTSetup = number;
type PFFFTDSetup = number;
type F32Ptr = number;
type F64Ptr = number;
export interface Exports extends WebAssembly.Exports {
    memory: WebAssembly.Memory;
    _initialize(): void;
}
interface Pffft32Exports extends Exports {
    pffft_new_setup(n: number, transform: ValueType): PFFFTSetup;
    pffft_destroy_setup(setup: PFFFTSetup): void;
    pffft_transform(setup: PFFFTSetup, input: F32Ptr, output: F32Ptr, work: F32Ptr, direction: Direction): void;
    pffft_transform_ordered(setup: PFFFTSetup, input: F32Ptr, output: F32Ptr, work: F32Ptr, direction: Direction): void;
    pffft_zreorder(setup: PFFFTSetup, input: F32Ptr, output: F32Ptr, direction: Direction): void;
    pffft_zconvolve_accumulate(setup: PFFFTSetup, dftA: F32Ptr, dftB: F32Ptr, dftAB: F32Ptr, scaling: number): void;
    pffft_zconvolve_no_accu(setup: PFFFTSetup, dftA: F32Ptr, dftB: F32Ptr, dftAB: F32Ptr, scaling: number): void;
    pffft_simd_size(): number;
    pffft_min_fft_size(transform: ValueType): number;
    pffft_is_valid_size(n: number, transform: ValueType): boolean;
    pffft_nearest_transform_size(n: number, transform: ValueType, higher: boolean): number;
    pffft_aligned_malloc(size: number): F32Ptr;
    pffft_aligned_free(ptr: F32Ptr): void;
}
interface Pffft64Exports extends Exports {
    pffftd_new_setup(n: number, transform: ValueType): PFFFTDSetup;
    pffftd_destroy_setup(setup: PFFFTDSetup): void;
    pffftd_transform(setup: PFFFTDSetup, input: F64Ptr, output: F64Ptr, work: F64Ptr, direction: Direction): void;
    pffftd_transform_ordered(setup: PFFFTDSetup, input: F64Ptr, output: F64Ptr, work: F64Ptr, direction: Direction): void;
    pffftd_zreorder(setup: PFFFTDSetup, input: F64Ptr, output: F64Ptr, direction: Direction): void;
    pffftd_zconvolve_accumulate(setup: PFFFTDSetup, dftA: F64Ptr, dftB: F64Ptr, dftAB: F64Ptr, scaling: number): void;
    pffftd_zconvolve_no_accu(setup: PFFFTDSetup, dftA: F64Ptr, dftB: F64Ptr, dftAB: F64Ptr, scaling: number): void;
    pffftd_simd_size(): number;
    pffftd_min_fft_size(transform: ValueType): number;
    pffftd_is_valid_size(n: number, transform: ValueType): boolean;
    pffftd_nearest_transform_size(n: number, transform: ValueType, higher: boolean): number;
    pffftd_aligned_malloc(size: number): F64Ptr;
    pffftd_aligned_free(ptr: F64Ptr): void;
}
type PffftExportsType<A> = A extends Float32Array ? Pffft32Exports : Pffft64Exports;
declare class Buffer<A extends FloatArray, T extends ValueType> implements Disposable {
    readonly type: T;
    readonly array: A;
    readonly _exports: A extends Float32Array ? Pffft32Exports : Pffft64Exports;
    constructor(arrayType: FloatArrayType<A>, n: number, type: T, exports: PffftExportsType<A>);
    [Symbol.dispose](): void;
    get ptr(): number;
    get _memory(): ArrayBuffer;
    get ctor(): FloatArrayType<A>;
}
export declare class Setup<A extends FloatArray, T extends ValueType> implements Disposable {
    #private;
    readonly arrayType: FloatArrayType<A>;
    readonly n: number;
    readonly work: Buffer<A, T>;
    constructor(arrayType: FloatArrayType<A>, type: T, n: number, exports: PffftExportsType<A>);
    [Symbol.dispose](): void;
    get type(): ValueType;
    makeSignalBuffer(extra?: number): Buffer<A, T>;
    makeSpectrumBuffer(extra?: number): Buffer<A, ValueType.COMPLEX>;
    makeInternalLayoutBuffer(extra?: number): Buffer<A, ValueType.REAL>;
    forward(signal: Buffer<A, T>, spectrum: Buffer<A, ValueType.COMPLEX>): void;
    inverse(spectrum: Buffer<A, ValueType.COMPLEX>, signal: Buffer<A, T>): void;
    forwardToInternalLayout(signal: Buffer<A, T>, spectrum: Buffer<A, ValueType.REAL>): void;
    inverseFromInternalLayout(spectrum: Buffer<A, ValueType.REAL>, signal: Buffer<A, T>): void;
    reorder(spectrum: Buffer<A, ValueType.REAL>, output: Buffer<A, ValueType.COMPLEX>): void;
    convolve(a: Buffer<A, ValueType.REAL>, b: Buffer<A, ValueType.REAL>, ab: Buffer<A, ValueType.REAL>, scaling: number): void;
    convolveAccumulate(a: Buffer<A, ValueType.REAL>, b: Buffer<A, ValueType.REAL>, ab: Buffer<A, ValueType.REAL>, scaling: number): void;
}
export declare class FFTLibrary<A extends FloatArray, T extends ValueType> {
    #private;
    readonly arrayType: FloatArrayType<A>;
    readonly type: T;
    constructor(arrayType: FloatArrayType<A>, type: T, exports: PffftExportsType<A>);
    setup(n: number): Setup<A, T>;
    nearestValidSize(n: number, higher: boolean): number;
    isValidSize(n: number): boolean;
    minFftSize(): number;
}
export declare function fetchWasm<E extends Exports>(url: URL, importObject?: WebAssembly.Imports): Promise<E>;
export declare function loadLibrary<A extends FloatArray, T extends ValueType>(arrayType: FloatArrayType<A>, type: T): Promise<FFTLibrary<A, T>>;
export {};
